# 第三方库
import time, json, pymysql, traceback
import pandas as pd
import numpy as np
from math import ceil
from copy import deepcopy
# 自发布的pip库
from .pmysql import pmysql



class 过滤器():

    def __init__(self, name=None, 条件=None, orm=None, 查询字段='*'):
        self.name = name
        self.条件 = 条件
        self.orm = orm
        self.查询字段 = 查询字段

    def copy(self, 条件, 查询字段=None):
        return 过滤器(name=self.name, 条件=条件, orm=self.orm, 查询字段=查询字段 or self.查询字段)

    def __eq__(self, o):  # ==
        if o is None:
            return self.copy(f"{self.name} is null")
        return self.copy(f"{self.name} = {json.dumps(o, ensure_ascii=False)}")

    def __ne__(self, o):  # !=
        if o is None:
            return self.copy(f"{self.name} is not null")
        return self.copy(f"{self.name} != {json.dumps(o, ensure_ascii=False)}")

    def __lt__(self, o):  # <
        return self.copy(f"{self.name} < {json.dumps(o, ensure_ascii=False)}")

    def __le__(self, o):  # <=
        return self.copy(f"{self.name} <= {json.dumps(o, ensure_ascii=False)}")

    def __gt__(self, o):  # >
        return self.copy(f"{self.name} > {json.dumps(o, ensure_ascii=False)}")

    def __ge__(self, o):  # >=
        return self.copy(f"{self.name} >= {json.dumps(o, ensure_ascii=False)}")

    def __and__(self, o):  # &
        if self.条件: return self.copy(f"({self.条件}) and ({o.条件})")
        return o

    def __or__(self, o):  # |
        if self.条件: return self.copy(f"({self.条件}) or ({o.条件})")
        return o

    def __invert__(self):  # ~
        if self.条件: return self.copy(f"not ({self.条件})")
        return self

    def re_m(self, o=''):
        if o: return self.copy(f"{self.name} regexp {json.dumps(o, ensure_ascii=False)}")
        raise Exception("正则内容不能为空")
    
    def __getitem__(self, key):
        type_ = type(key)
        if type_ is slice:
            start, stop, step = key.start or 0, key.stop or 0, key.step
            if (start < 0) or (stop < 0): raise Exception("不支持负数切片")
            elif start and stop: limit = f" limit {start}, {stop - start}"
            elif start: limit = f" limit {start}, 9999999999999999"
            elif stop: limit = f" limit 0, {stop}"
            else: limit = ""
            sql = f"select {self.查询字段}" + " from {数据表}" + (f" where {self.条件}" if self.条件 else '') + limit
            return self.orm.sql(sql)
        elif type_ is int:  # [3]
            if key < 0: raise Exception("不支持负数切片")
            sql = f"select {self.查询字段}" + " from {数据表}" + (f" where {self.条件}" if self.条件 else '') + f" limit {key}, 1"
            r = self.orm.sql(sql)
            return r[0]
        elif type_ is str:  # 查询单个字段
            return self.copy(self.条件, key)
        elif type_ is dict:  # 字段重命名
            查询字段 = ','.join(f"{k} as {v}" for k, v in key.items())
            return self.copy(self.条件, 查询字段)
        elif type_ is tuple:  # 部分字段重命名
            查询字段 = []
            for x in key:
                if type(x) is str: 查询字段.append(x)
                else:
                    for k, v in x.items(): 查询字段.append(f"{k} as {v}")
            查询字段 = ','.join(查询字段)
            return self.copy(self.条件, 查询字段)
        elif self.条件:  # [过滤器]
            return self.copy(f"({self.条件}) and ({key.条件})")
        return key  # [过滤器]

    def __setitem__(self, key, value):
        type_ = type(key)
        if type_ is slice:
            raise Exception("不支持切片修改, 仅支持条件修改")
        elif type_ is int:  # [3]
            raise Exception("不支持切片修改, 仅支持条件修改")
        zds = self.orm.s__zds if (self.查询字段 == '*') else self.查询字段.split(',')
        if type_ is str:
            zds = [key]
            条件 = f" where {self.条件}" if self.条件 else ''
        elif type_ is tuple:
            zds = list(key)
            条件 = f" where {self.条件}" if self.条件 else ''
        elif self.条件:
            条件 = f" where ({self.条件}) and ({key.条件})"
        else:
            条件 = f" where {key.条件}"
        pk = self.orm.s__pk
        if (pk not in value) and (pk in zds): zds.remove(pk)
        value = {k: value.get(k) for k in zds}
        value = ','.join([f"{k}={json.dumps(v, ensure_ascii=False)}" for k, v in value.items()])
        sql = "update {数据表}" + f" set {value}" + 条件
        return self.orm.sql(sql)

    def get_sql(self):
        return self.条件

    def delete(self):
        sql = "delete from {数据表}" + f" where {self.条件}"
        return self.orm.sql(sql=sql)

    def str_in(self, o=[]):
        条件null = 条件in = None
        o = set(o)
        if None in o:
            o.remove(None)
            条件null = f"{self.name} is null"
        for x in o:
            if type(x) is not str:
                raise Exception("str_in列表只接收str或None")
        if o: 条件in = f"{self.name} in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
        if 条件null and 条件in:
            return self.copy(f"({条件null}) or ({条件in})")
        elif 条件null:
            return self.copy(条件null)
        elif 条件in:
            return self.copy(条件in)
        else:
            raise Exception("str_in列表不能为空")
            
    def str_not_in(self, o=[]):
        条件null = 条件in = None
        o = set(o)
        if None in o:
            o.remove(None)
            条件null = f"{self.name} is not null"
            if o:
                for x in o:
                    if type(x) is not str: raise Exception("str_not_in列表只接收str或None")
                条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
                return self.copy(f"({条件null}) and ({条件in})")
            return self.copy(条件null)
        elif o:
            for x in o:
                if type(x) is not str: raise Exception("str_not_in列表只接收str或None")
            条件null = f"{self.name} is null"
            条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
            return self.copy(f"({条件null}) or ({条件in})")
        raise Exception("str_not_in列表不能为空")

    
    def int_in(self, o=[]):
        条件null = 条件in = None
        o = set(o)
        if None in o:
            o.remove(None)
            条件null = f"{self.name} is null"
        for x in o:
            if type(x) is not int: raise Exception("int_in列表只接收int或None")
        if o: 条件in = f"{self.name} in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
        if 条件null and 条件in:
            return self.copy(f"({条件null}) or ({条件in})")
        elif 条件null:
            return self.copy(条件null)
        elif 条件in:
            return self.copy(条件in)
        else:
            raise Exception("int_in列表不能为空")
    
    def int_not_in(self, o=[]):
        条件null = 条件in = None
        o = set(o)
        if None in o:
            o.remove(None)
            条件null = f"{self.name} is not null"
            if o:
                for x in o:
                    if type(x) is not int: raise Exception("int_not_in列表只接收int或None")
                条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
                return self.copy(f"({条件null}) and ({条件in})")
            return self.copy(条件null)
        elif o:
            for x in o:
                if type(x) is not int: raise Exception("int_not_in列表只接收int或None")
            条件null = f"{self.name} is null"
            条件in = f"{self.name} not in ({','.join(json.dumps(x, ensure_ascii=False) for x in o)})"
            return self.copy(f"({条件null}) or ({条件in})")
        raise Exception("int_not_in列表不能为空")


class orm_mysql(pmysql):

    '''

<<使用教程>>


一, 连接数据库


地址 = {
    '账号': {'host': '', 'user': '', 'passwd': '', 'port': 3306},
    '数据库': '数据库名称',
    '数据表': '数据表名称',
}
orm = orm_mysql(地址=地址, mysql账号=None, 持久连接=True)
持久连接=True时, 每次增删改查的速度比 持久连接=False 快0.5秒, 但用完后需要手动调用 self.close()
持久连接=False时, 每次执行增删改查, 都会创建一个临时的mysql连接, 用完后立即自动关闭, 所以需要多0.5秒


二, 添加数据


kw = {'title':xxx, 'content':xxx}

orm.append(kw)  # 添加一条数据

orm.数据分组尺寸 = 5000  # 默认为1000, 数值越大上传越快, 但mysql本身不支持无限大的分组尺寸
orm += [kw1, kw2, kw3]  # 批量添加数据


三, 过滤器[ ]


1.通过过滤器[ ]查询

查询所有数据: orm[:]

查询 id>=10 的数据: orm[orm.id >= 10][:]

查询 id==10 的数据: orm[orm.id == 10][:]

查询 title=='t' 的数据: orm[orm.title == 't'][:]

用正则表达式查询: orm[orm.title.re_m('.*')][:]

查询id在(1,3,5)里面的数据: orm[orm.id.int_in([1,3,5])][:]

查询title在('t1','t2')里面的数据: orm[orm.title.str_in(['t1','t2'])][:]

查询id不在(1,3,5)里面的数据: orm[orm.id.int_not_in([1,3,5])][:]

查询title不在('t1','t2')里面的数据: orm[orm.title.str_not_in(['t1','t2'])][:]

并联查询:
orm[(orm.id > 5) & (orm.title.re_m('.*'))][:]  # and
orm[(orm.id > 5) | (orm.title.re_m('.*'))][:]  # or
orm[~ (orm.id > 5)]  # 将条件取反
orm[() & (~ (() | (() & ()) | ()))][:]  # 可无限嵌套

串联查询
orm[orm.id > 5][orm.title == 't'][][][]...[:]  # 可无限嵌套

并联和串联混合查询
orm[并联语句][并联语句][并联语句][并联语句]...[:]

# 只查询某几列
orm[一系列过滤器]['title'][:]  # 只查询title字段
orm[一系列过滤器]['title', 'id'][:]  # 只查询title和id字段
orm[一系列过滤器]['title', {'id':'pk'}][:]  # 只查询title和id字段, 并且把id重命名为pk

# 切片
查询全部: orm[一系列过滤器][:]
查询前100条: orm[一系列过滤器][:100]
查询第101条开始的: orm[一系列过滤器][100:]
查询第100条: orm[一系列过滤器][100]


2.通过过滤器[ ]删除数据

orm[一系列过滤器].delete()
orm[orm.id >= 10].delete()


3.通过过滤器[ ]修改数据

kw = dict(title=xx, content=xx, 相似度=xx, ....)
orm[一系列过滤器] = kw  # 对所有字段赋值
orm[一系列过滤器]['title'] = kw  # 仅对title字段赋值
orm[一系列过滤器]['title','content'] = kw  # 仅对title和content字段赋值


四, loc 和 ilic


# 按标签切片
kws = orm.loc[5:]  # 等价于 kws = orm[orm.id >= 5][:]
kws = orm.loc[:5]  # 等价于 kws = orm[orm.id <= 4][:]
kws = orm.loc[:]   # 等价于 kws = orm[:]
kw = orm.loc[5]    # 等价于 kw = orm[orm.id == 5][0]
orm.loc[5] = kw    # 等价于 orm[orm.id == 5] = kw

# 按位置切片
kws = orm.iloc[5:]  # 等价于 kws = orm[5:]
kws = orm.iloc[:5]  # 等价于 kws = orm[:5]
kws = orm.iloc[:]   # 等价于 kws = orm[:]
kw = orm.iloc[5]    # 等价于 kw = orm[5]

'''
    
    添加失败立即报错 = True
    数据分组尺寸 = 1000

    def __init__(self, 地址=None, mysql账号={}, 持久连接=True):
        self.添加失败的数据 = []
        if mysql账号: self.mysql账号 = mysql账号  # 1
        self.s__构建地址(地址)  # 2
        self.持久连接 = 持久连接  # 3
        if 持久连接: self.创建持久连接()  # 3
        self.获取字段列表()  # 4
        self.loc = self.type_loc(self)  # 5
        self.iloc = self.type_iloc(self)  # 5
        for name in self.s__zds:  # 5
            exec(f"self.{name} = 过滤器(name='{name}', orm=self)")
        else:
            self.s__过滤器 = eval(f"self.{name}")

    def s__构建地址(self, o):
        if type(o) is str:
            self.地址 = self.mysql账号[o]
        else:
            def get账号(o):
                r = {}
                for k, v in o.items():
                    if type(v) is dict:
                        r.update(get账号(v))
                    else:
                        r[k] = v
                return r
            地址 = get账号(o)
            地址['账号'] = {'host': 地址.pop('host'), 'user': 地址.pop('user'), 'passwd': 地址.pop('passwd'), 'port': 地址.pop('port')}
            self.地址 = 地址

    def __getitem__(self, key):
        return self.s__过滤器.__getitem__(key)

    def __setitem__(self, key, value):
        return self.s__过滤器.__setitem__(key, value)

    def 创建持久连接(self): self.conn, self.cursor = self.创建连接(地址=self.地址)

    def close(self): return self.conn.close()

    def sql(self, sql=None, **kw):
        数据类型 = kw.get('数据类型', 'list').lower()
        地址 = self.地址
        if self.持久连接:
            conn, cursor = self.conn, self.cursor
            conn.ping(reconnect=True)
            # 加一步判断: 断开的话, 自动重连
        else:
            conn, cursor = self.创建连接(地址=地址)
        if sql:
            sql = sql.replace('{数据表}', 地址['数据表'])
            cursor.execute(sql)
            datas = list(cursor.fetchall())
            conn.commit()
            if not self.持久连接: conn.close()
            if 数据类型 in ('pandas', 'df'): return pd.DataFrame(datas)
            return datas
        return conn, cursor, 地址['数据表']

    def append(self, kw):  # 同list: orm.append({'title': '2020-02-25'})
        return self.添加数据(datas=kw)
    
    def __iadd__(self, kws):  # 同list: orm += [{'title': '2020-02-25-0'}, {'title': '2020-02-25-1'}]
        r = self.添加数据(datas=kws)
        print(r)
        return self

    def 获取字段列表(self):
        sql = "select column_name as name,column_key as pk from information_schema.columns where table_name = '{数据表}'"
        kws = self.sql(sql=sql)
        self.s__zds = 全部字段 = []
        self.s__pzds = 普通字段 = []
        self.s__pk = ''
        实例变量 = set(self.__dict__) | {'s__过滤器', 'loc', 'iloc'}
        for x in kws:
            name = x['name']
            if name in 实例变量:
                raise Exception(f"数据表字段 '{name}' 名称与orm内置变量冲突, 无法使用orm")
            全部字段.append(name)
            if x['pk']:
                self.s__pk = name
            else:
                普通字段.append(name)

    class type_loc():
        def __init__(self, f):
            self.s__pk = f.s__pk
            self.sql = f.sql
            self.修改数据 = f.修改数据

        def __getitem__(self, key):  # 同pandas的loc orm.loc[5], orm.loc[5:], orm.loc[:5]
            if type(key) is slice:
                start, stop, step = key.start, key.stop, key.step
                if start and stop:
                    sql = "select * from {数据表}" + f" where ({self.s__pk} >= {start}) and ({self.s__pk} < {stop})"
                elif start:
                    sql = "select * from {数据表}" + f" where {self.s__pk} >= {start}"
                elif stop:
                    sql = "select * from {数据表}" + f" where {self.s__pk} < {stop}"
                else:
                    sql = "select * from {数据表}"
                return self.sql(sql=sql)
            else:
                key = json.dumps(key, ensure_ascii=False)
                sql = "select * from {数据表}" + f" where {self.s__pk} = {key}"
                r = self.sql(sql=sql)
                return r[0]
        
        def __setitem__(self, key, value):  # 同pandas的loc orm.loc[5] = dict(title='dad')
            if type(key) is slice:
                print('暂不支持切片赋值, 只支持索引赋值')
                return []
            else:
                value = deepcopy(value)
                value[self.s__pk] = key
                return self.修改数据(代号=None, datas=[value], pro_key=self.s__pk)
    
    class type_iloc():
        def __init__(self, f):
            self.s__pk = f.s__pk
            self.sql = f.sql

        def __getitem__(self, key):  # 同pandas的iloc orm.iloc[5], orm.iloc[5:], orm.iloc[:5]
            if type(key) is slice:
                start, stop, step = key.start or 0, key.stop or 0, key.step
                if (start < 0) or (stop < 0): raise Exception("不支持负数切片")
                if start and stop:
                    sql = "select * from {数据表}" + f" limit {start}, {stop-start}"
                elif start:
                    sql = "select * from {数据表}" + f" limit {start}, 9999999999999999"
                elif stop:
                    sql = "select * from {数据表}" + f" limit 0, {stop}"
                else:
                    sql = "select * from {数据表}"
                return self.sql(sql=sql)
            else:
                if key < 0: raise Exception("不支持负数切片")
                key = json.dumps(key, ensure_ascii=False)
                sql = "select * from {数据表}" + f" limit {key}, 1"
                r = self.sql(sql=sql)
                return r[0]


    def 添加数据(self, datas, 极致=True):
        conn, cursor, 数据表 = self.sql()
        if type(datas) is dict:
            try:
                字段s = set(datas)
                sql = f"insert into {数据表}({', '.join(字段s)}) VALUES ({', '.join(('%s',)*len(字段s))})"
                cursor.execute(sql, tuple(datas.values()))
                conn.commit()
            except Exception as err:
                print(f"{err} - {datas}")
                self.添加失败的数据.append(deepcopy(datas))
                if not self.持久连接:
                    try: conn.close()
                    except: pass
                if self.添加失败立即报错:
                    raise Exception("存在添加失败的数据, 通过 orm.添加失败的数据 查看")
                return False
            if not self.持久连接:
                try: conn.close()
                except: pass
            return True
        else:
            size = self.数据分组尺寸
            fails = []
            df = pd.DataFrame(datas)
            字段s = set(df.columns)
            datas = json.loads(df.to_json(orient='records'))
            df = None
            # 初始化数据库连接
            sql = f"insert into {数据表}({', '.join(字段s)}) VALUES ({', '.join(('%s',)*len(字段s))})"
            # 批量上传
            datas = [datas[size*(i-1): size*i] for i in range(1, ceil(len(datas)/size)+1)]
            group_number = len(datas)
            for i, idatas in enumerate(datas):
                print(f'\r正在批量上传第 {i+1}/{group_number} 组, 每组 {size} 条', end='    ')
                values = tuple([tuple(x.values()) for x in idatas])
                try:
                    cursor.executemany(sql, values)
                    conn.commit()
                except Exception as err:
                    conn.rollback()  # 回滚
                    for x in idatas: fails.append(x)
                    print(f'本组插入失败, 已回滚 - {err}')
            # 逐条上传
            if 极致 and fails:
                print('\n已开启极致模式, 开始对批量上传失败的进行逐条上传')
                fails, datas, total = [], fails, len(fails)
                for count, data in enumerate(datas):
                    print(f'\r正在逐条上传第 {count+1}/{total} 条', end='    ')
                    try: cursor.execute(sql, tuple(data.values()))
                    except Exception as err:
                        fails.append(data)
                        print(f"{err} - {data}    ")
                    if (count+1) % size == 0: conn.commit()
                conn.commit()
            # 关闭数据库连接
            try:
                if not self.持久连接: conn.close()
            except: pass
            print(f'\n\n上传完成, 上传失败共 {len(fails)} 条')
            if fails:
                添加失败 = self.添加失败的数据
                for x in fails: 添加失败.append(deepcopy(x))
                if self.添加失败立即报错:
                    raise Exception("存在添加失败的数据, 通过 orm.添加失败的数据 查看")
                return False
            return True



import jieba, re
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

class phk:

    class 数据类型:
        def time():
            from phk.data_type import 数据类型
            return 数据类型.time
    
    class 善检网:
        class 供应链:
            def api():
                from phk.shanjianwang_gongyinglian_api import 供应链api
                return 供应链api
        class api第2版():
            class 财务():
                def api():
                    from phk.shanjianwang_api2_caiwu import 善检网api第2版_财务
                    return 善检网api第2版_财务
            class 套餐():
                def 腾讯():
                    from phk.shanjianwang_api2_taocan_tengxun import 善检网api第2版_套餐_腾讯
                    return 善检网api第2版_套餐_腾讯
                def 阿里():
                    from phk.shanjianwang_api2_taocan_ali import 善检网api第2版_套餐_阿里
                    return 善检网api第2版_套餐_阿里
                def 百度():
                    from phk.shanjianwang_api2_taocan_baidu import 善检网api第2版_套餐_百度
                    return 善检网api第2版_套餐_百度

    class 站群系统:
        class 设置相似文章:
            def 在相同分类中推荐():
                from phk.zhanqun_shezhi_xiangsi_wenzhang import 设置相似文章
                return 设置相似文章

    class 算法:
        def dfa():
            from phk.dfa import dfa
            return dfa

    class webtool:
        def tool():  # 网络操作工具
            from phk.webtool import webtool
            return webtool
        
    class 爬虫:
        def 基础爬虫():
            from phk.req_base import req_base
            return req_base
        
        class 百度:
            def pc():
                from phk.req_baidu_pc import req_baidu_pc
                return req_baidu_pc

    class seo:
        class 百度:
            def pc():
                from phk.seo_baidu_pc import seo_baidu_pc
                return seo_baidu_pc

    class localtool:
        
        def tool():  # 本地操作工具
            from phk.tool import tool
            return tool

        def 装饰器():
            from phk.deco import 装饰器
            return 装饰器
        
        def 自动任务():
            from phk.autotask import 自动任务
            return 自动任务
    
    class 第三方平台:

        class 百度:
            class ai开放平台:
                def 百度aip():
                    from phk.baidu_aip import baidu_aip
                    return baidu_aip

        class 计费系统:
            class 起点:
                def api():
                    from phk.gexitong_jifeixitong_qidian_api import 起点计费系统api
                    return 起点计费系统api
                def 爬虫():
                    from phk.gexitong_jifeixitong_qidian_pachong import 起点系统爬虫
                    return 起点系统爬虫
                def 代理商关键词():
                    from phk.gexitong_jifeixitong_qidian_pachong import 代理商关键词
                    return 代理商关键词
    
    class 数据库:
        class mysql:
            def orm2():
                from phk.orm_mysql_2 import orm_mysql
                return orm_mysql

            def tool():
                from phk.pmysql import pmysql
                return pmysql

            def orm():
                from phk.orm_mysql import orm_mysql
                print('orm 模仿 pandas 的过滤器, 因此也延续了pandas的重大缺陷')
                print('此版本的orm已停止维护, 导入 phk.数据库.mysql.orm2() 使用最新版本的orm')
                return orm_mysql

    # 兼容张工代码
    def 字符串排列(text, long=3, 不拆分=[]):
        # long: 返回时, 将排除 len(x) < long 的组合
        # 不拆分: 作为一个不可分割的最小字符, 不进一步拆分的词
        for x in 不拆分: jieba.add_word(x)
        text = ''.join(re.findall('[\u4e00-\u9fa5a-zA-Z]', text))  # 去掉所有空格和符号
        kws = list(set(jieba.cut(text)))
        total = len(kws)
        # 排列组合
        list_all = kws.copy()
        for i1 in range(0, total-1):
            for i2 in range(i1+1, total):
                list_all.append(kws[i1] + kws[i2])
        # 去重和按字数过滤
        list_all = [x for x in set(list_all) if len(x) >= long]
        return list_all

    # 兼容张工代码
    def sqlalchemy执行sql(mysql_url, sqls):
        engine = create_engine(mysql_url, encoding='utf-8')  # 创建的数据库引擎
        DBSession = sessionmaker(bind=engine)  #创建session类型
        session = DBSession()  #创建session对象，进行增删改查:
        rs = {}
        for sql in sqls:
            r = session.execute(sql)
            try:
                rs[sql] = [dict(zip(result.keys(), result)) for result in r]
                print(f'{sql} --> done')
            except Exception as err:
                rs[sql] = str(err)
                print(f'{sql} --> {err}')
        session.commit()
        session.close()
        return rs



# -!- coding: utf-8 -!-
import re, os, sys, requests, json, math, time, copy, random, traceback
import pandas as pd
from math import ceil
from .data_set import 数据集

全国城市字典 = 数据集.全国城市.获取城市字典()

class 善检网api第2版_财务():
    domain = 'https://mc.tijian8.com'
    账号, 密码 = '', ''
    
    def __init__(self, 账号=None, 密码=None):
        if 账号: self.账号 = 账号
        if 密码: self.密码 = 密码
        self.更新token()
        self.初始化()
    
    def 初始化(self):
        self.字段清单 = [
            ('oid', 'oid', 'oid'),
            ('sql_state', 'sql_state', 'sql_state'),

            ('城市', 'city', 'city'),
            ('城市', 'city', 'cityName'),
            ('csid', 'csid', 'csid'),
            ('医院', 'hospital', 'hospitalName'),
            ('医院', 'hospital', 'bookingMerchant'),
            ('医院简称', 'hospital_jc', None),
            
            ('esid', 'esid', 'esid'),
            ('机构', 'org', 'bookingMEC'),
            ('机构', 'org', 'orgName'),
            ('机构类型', 'org_type', 'orgType'),
            ('机构类型', 'org_type', 'typeName'),

            ('用户名', 'user_account', 'customerAct'),
            ('下单时间', 'order_time', 'orderTime'),
            ('订单来源', 'order_origin', 'orderOrigin'),
            ('订单类型', 'order_type', 'orderType'),

            ('支付状态', 'pay_status', 'payStatus'),
            ('支付状态', 'pay_status', 'orderStatus'),
            ('支付方式', 'pay_type', 'payType'),
            ('认领状态', 'claim_status', 'claimStatus'),
            ('认领客服', 'kefu', 'claimName'),
            ('订单原价', 'origin_price', 'originPrice'),
            ('订单金额', 'order_price', 'orderPrice'),
            ('订单实付款', 'payed_price', 'payedPrice'),
            ('订单合同结算价', 'order_contract_settlement', 'contractSettlementPrice'),
            ('订单结算价', 'order_settlement', 'actSettlementPrice'),
            ('订单结算价', 'order_settlement', 'settlementPrice'),
            ('订单利润', 'order_profit', None),
            ('订单合同结算时间', 'contract_settlement_time', 'contractSettlementTime'),
            ('订单结算完成时间', 'finally_settlement_time', 'finallySettlementTime'),

            ('rid', 'rid', 'rid'),
            ('年龄', 'age', 'bookingAge'),
            ('性别', 'gender', 'bookingGender'),
            ('套餐名称', 'goods_name', 'bookingGoods'),
            ('套餐原价', 'goods_origin_price', 'goodsOriginPrice'),
            ('套餐售价', 'goods_sale_price', 'goodsSalePrice'),
            ('套餐结算价', 'goods_settlement_price', 'goods_settlement_price'),
            ('套餐利润', 'goods_profit', None),
            ('预约状态', 'booking_status', 'bookingStatus'),
            ('预估结算价', 'predict_settlement', 'presettlePrice'),
            ('预估利润', 'predict_profit', None) 
        ]

        self.字段中文清单 = list(set([a for a,b,c in self.字段清单]))

        self.字段字典 = {b: str(dict(zip(['中文', '数据库', 'api'], a))) for a in self.字段清单 for b in a}

        self.代号值字典 = dict(
            订单类型 = {2:'普通订单', 5:'大型检查订单', 6:'简版商城订单', 100:'线下订单'},
            订单来源 = {1:'PC', 2:'WAP', 3:'小程序', 5:'线下'},
            支付状态 = {0:'未付款', 2:'已付款', 3:'待退款', 4:'已退款', 5:'部分退款'},
            认领状态 = {0:'未认领', 1:'已认领'},
            支付方式 = {1:'微信支付', 2:'支付宝WAP', 3:'威付通微信', 4:'威付通支付宝', 5:'威付通银联', 6:'支付宝PC'},
            预约状态 = {0:'审核中', 1:'审核通过', 2:'审核驳回', 3:'体检完成', 4:'取消预约', 5:'大型体检未支付', 6:'已结算',
                   7:'申请结算', 8:'已预付', 9:'申请预付'}
        )
    
    def 转为中文(self, item):
        if type(item) is dict:
            for a, c in [(a,c) for a,b,c in self.字段清单 if c in item]:
                item[a] = item.pop(c)
            for key, item_b in self.代号值字典.items():
                if key in item:
                    value = item[key]
                    if value in item_b:
                        item[key] = item_b[value]
        return item

    def 保留小数(self, item, n):
        for k,v in item.items():
            if type(v) is float:
                item[k] = round(v, n)
        return item

    def 更新token(self):
        url = f'{self.domain}/login'
        headers = {'Content-Type': 'application/json'}
        data = json.dumps({"account": self.账号, "password": self.密码})
        r = requests.post(url, data=data, headers=headers)
        self.token = r.json()['data']['token']

    def 请求服务器(self, 方法, api, data=None, params=None):
        url = f'{self.domain}/{api}'
        data = json.dumps(data) if data else None
        for n in range(3):
            headers = {'Content-Type': 'application/json', 'Authorization': f"attackFind:{self.token}"}
            if 方法 == 'get':
                r = requests.get(url, data=data, params=params, headers=headers).json()
            elif 方法 == 'post':
                r = requests.post(url, data=data, headers=headers).json()
            elif 方法 == 'put':
                r = requests.put(url, data=data, params=params, headers=headers).json()
            if r['status'] < 1000:
                break
            else:
                self.更新token()
        return r
    
    def 查询订单列表(self, params=None):
        api = "finance/orders"
        p = params.copy()
        p['per'] = 1
        r = self.请求服务器('get', api, params=p)
        kws = []
        p['per'] = min(1000, r['total'])
        pagetotal = ceil(r['total'] / 1000)
        for page in range(1, pagetotal + 1):
            print(f"\r查询订单列表 第 {page}/{pagetotal} 页", end='    ')
            p['page'] = page
            for data in self.请求服务器('get', api, params=p)['data']:
                self.转为中文(data)
                try: data['订单利润'] = data['订单实付款'] - data['订单结算价']
                except: data['订单利润'] = None
                self.保留小数(data, 2)
                kws.append(data)
        return kws

    def 查询订单详情(self, params):
        api = 'finance/order-detail'
        for n in range(30):  # 重试15次
            try:
                r = self.请求服务器('get', api, params=params)
                break
            except Exception as err:
                e = err
                if 'Max retries exceeded with url' in str(e): time.sleep(1)
                else: raise e
        else: raise e
        data = r['data']
        self.转为中文(data)
        data.pop('订单结算价')
        for item in data['bookingInformation']:
            item['goods_settlement_price'] = item.pop('settlementPrice')
            self.转为中文(item)
            try: item['预估利润'] = item['套餐售价'] - item['预估结算价']
            except: pass
            try: item['套餐利润'] = item['预估利润'] = item['套餐售价'] - item['套餐结算价']
            except: pass
            try: item['性别'] = '女' if int(item['bookingIdcard'][16]) % 2 == 0 else '男'
            except: item['性别'] = '未知'
            self.保留小数(item, 2)
        self.保留小数(data, 2)
        return r
    
    def 获取商务机构信息(self, params={}):  # 没问题
        api = 'commercial/hospital-list'
        params['per'] = self.请求服务器('get', api, params=params)['total']
        r = self.请求服务器('get', api, params=params)
        data = r['data']
        item = {
            'businessDirector': '商务负责人',
            'channel': '渠道类型',
             'holdName': '商务录入人',
             'operateTime': '操作时间',
             'settlementDiscount': '结算折扣',
             'mecname': '机构名称',
             'settlementPeriod': '结算周期',
             'meclevel': '机构等级',
             'mecstatus': '机构状态',
            'provinceName': '省', 'cityName': '市', 'areaName': '区',
            'notice':'备注'
        }

        机构等级 = {'glsj':'公立三甲', 'glyy':'公立医院', 'mysj':'民营三级', 'myyy':'民营医院', 'sjjd':'民营三甲', 'zyjg':'专业机构'}
        机构状态 = {0:'未审核' , 1:'审核通过合作中' , 2:'暂停' ,3:'驳回' ,4:'逻辑删除'}
        渠道类型 = {1:'直签-机构', 2:'直签-个人', 3:'第三方-公司' ,4:'第三方-个人'}
        df = pd.DataFrame(data).rename(columns=item)
        df['机构等级'] = df.机构等级.apply(lambda x: 机构等级.get(x, x))
        df['机构状态'] = df.机构状态.apply(lambda x: 机构状态.get(x, x))
        df['渠道类型'] = df.渠道类型.fillna('-')
        df['渠道类型'] = df.渠道类型.apply(lambda x: 渠道类型.get(x, x).split('-'))
        df['渠道1'] = df.渠道类型.apply(lambda x:x[0])
        df['渠道2'] = df.渠道类型.apply(lambda x:x[-1])
        df['城市等级'] = df.市.apply(lambda x: 全国城市字典.get(x, {}).get('等级', 6))
        df.drop('渠道类型', axis=1, inplace=True)
        df = df[~df['机构名称'].str.contains('test|测试')]
        
        品牌s = ['美亚', '美年', '中信健康', '艾诺', '福华', '慈铭', '新华', '民流健康', '美兆', '中信', '普惠', '国药阳光', '新华体检', '民众', '艾博', '熙康', '第一健康', '仁泰', '爱康', '瑞慈', 'PET', '医院']
        品牌s = '|'.join(set(品牌s))
        df['品牌'] = df.机构名称.apply(lambda x: (re.findall(品牌s, x) or ['其它'])[0])

        return df

    
    def 获取机构套餐(self, esid):
        api = 'util/hospital-products'
        r = self.请求服务器('get', api, params=dict(esid=esid))
        return r['data']
    
    def 获取省市区(self, 行政级别, 有机构有套餐=False):
        api = 'util/address'
        params = dict(dataType=行政级别, bizType= 2 if 有机构有套餐 else 1)
        r = self.请求服务器('get', api, params=params)
        for x in r['data']:
            x['行政级别'] = 行政级别
        return r['data']
    
    def 获取区域下的机构(self, 行政级别, 地区编号):
        api = 'util/address-hospitals'
        params = dict(dataType=行政级别, bizId=地区编号)
        r = self.请求服务器('get', api, params=params)
        return r['data']
    
    def 获取区号字典(self, key='区号'):  # key: 区号, 城市, 双向
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.146 Safari/537.36"}
        r = requests.get('https://www.tijian8.com/', timeout=30, headers=headers)
        quhaos = list(set(re.findall('<li ><a href="/(\d*)">(.*)</a></li>', r.text)))
        if key == '区号': return {a:b for a,b in quhaos}
        elif key == '城市': return {b:a for a,b in quhaos}
        elif key == '双向': return {x:y for a,b in quhaos for x,y in ((a,b), (b,a))}

    def 获取全部机构(self):
        orgs = self.获取区域下的机构(行政级别=None, 地区编号=None)
        orgs = [x for x in orgs if 'test' not in x['name']]
        城市字典 = {x['id']:x['title'] for x in self.获取省市区(2)}
        区号字典 = self.获取区号字典(key='城市')
        for x in orgs:
            x['city'] = city = 城市字典[x['city']]
            x['city_b'] = city = city.replace('市', '')
            x['区号'] = 区号字典.get(city)
        return orgs



# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    version = 'v8.7', #版本

    # 依赖包, 如果没有, 可以不要
    # baidu-aip包会报错, 需手动添加
    # 手动安装 scrapy 库
    requires = [
        'pandas', 'pymysql', 'requests', 'numpy', 'sqlalchemy', 'apscheduler'
    ],

    description = 'me', # 简要描述
    py_modules = [], # 需要打包的模块
    packages = ['phk'],
    name = 'phk',# 需要打包的名字, 即本模块要发布的名字
    author = 'jutooy', # 作者名
    author_email = 'jutooy@qq.com',   # 作者邮件
    url='https://www.yuque.com/jutooy/python/', # 项目地址,一般是代码托管的网站
    license = 'MIT'
)

# 步骤:
# 修改版本号
# python 删除旧文件.py
# push到码云
# python setup.py sdist bdist_wheel
# twine upload dist/*
#     账号: 
#     密码: 
# python 删除旧文件.py
# pip install --upgrade phk
